function pic = gen_image(type, width, num)
% type can be from [{star}, corner]. width = 1, num = 10 by default.

global C

if nargin < 3, num = 10; end
if nargin < 2, width = 1; end
if nargin < 1, type = 'star'; end

ctrs = rand(2,1) * (C.n_pixel - 5*width) + 2.5*width;
for i = 2:num
	flag = 0;
	while(flag < 6*width)
		x = rand() * (C.n_pixel - 5*width) + 2.5*width;
		y = rand() * (C.n_pixel - 5*width) + 2.5*width;
		flag = min([abs(ctrs(1,:)-x) abs(ctrs(2,:)-y)]);		
	end
	ctrs(1,i) = x;
	ctrs(2,i) = y;
end

mags = rand(1,num)*2 + 3;
thetas = rand(1,num)*360; % only used for type = 'corner'

if strcmpi(type, 'star')	
	pic = zeros(C.n_pixel);
	for star = [ctrs; mags]
		w = width; % (.1 + .9 * rand())*width;
		
		% poisson process
		nphotons = C.numphotons; % round(C.p_mag0 * 10^(-.4*star(3)));
		xs = round(randn(nphotons,1)*w + star(1));
		ys = round(randn(nphotons,1)*w + star(2));
		good = xs >=1 & xs <= C.n_pixel & ys >=1 & ys <= C.n_pixel;
		xs = xs(good);
		ys = ys(good);
		xrange = min(xs):max(xs); % max(min(floor(a-3*w):ceil(a+3*w), C.n_pixel), 1);
		yrange = min(ys):max(ys); %max(min(floor(b-3*w):ceil(b+3*w), C.n_pixel), 1);
		if numel(yrange) == 1
			if yrange == 1, yrange = [1 2];
			else yrange = [yrange-1 yrange];
			end
		end
		for x = xrange
			pic(x,yrange) = pic(x,yrange) + hist(ys(xs == x), yrange);
		end
 		
		%{
		a = star(1);
		b = star(2);
        for i = max(1, floor(a-3*w)):min(C.n_pixel, ceil(a+3*w))
			for j = max(1, floor(b-3*w)):min(C.n_pixel, ceil(b+3*w))
				pic(i,j) = pic(i,j) + C.psf((i-a)/w, (j-b)/w) / w.^2 * C.p_mag0 * 10^(-.4*star(3));
			end
		end
		norm(pic(max(1, floor(a-3*w)):min(C.n_pixel, ceil(a+3*w)), max(1, floor(b-3*w)):min(C.n_pixel, ceil(b+3*w))), 1)
		%}
	end
	% pic = round(pic);
end

if strcmpi(type, 'corner')
	pic = zeros(C.n_pixel);
	for corner = [ctrs; mags; thetas]
		w = round(width*corner(3));
		w5 = round(5*width*corner(3));
		shape = imrotate([ones(w, w5); [ones(w5-w,w) zeros(w5-w)]], corner(4), 'bilinear');
		loc = round(corner(1:2)' - size(shape)/2);
		if loc(1) < 0
			shape = shape(2-loc(1):end,:);
			loc(1) = 1;
		end
		if loc(2) < 0
			shape = shape(:,2-loc(2):end);
			loc(2) = 1;
		end
		if loc(1) + size(shape,1) > C.n_pixel
			shape = shape(1:1-loc(1)+C.n_pixel,:);
		end
		if loc(2) + size(shape,2) > C.n_pixel
			shape = shape(:,1:1-loc(2)+C.n_pixel);
		end
		
		xrange = loc(1):min(C.n_pixel,loc(1)+size(shape,1)-1);
		yrange = loc(2):min(C.n_pixel,loc(2)+size(shape,2)-1);
		pic(xrange, yrange) = max(pic(xrange,yrange), shape);
	end
end
