global C
global pyramid
startup;

% constants.m defines constants for the image
constants;

gen_pyramid;
disp('Loaded')

measurements = [100:200:700];  % Number of measurements to use

num_trials = 1;  %Loop num_trials times, plot the median result.

new_points = 1;  %Whether to generate new points when rerunning

% Another useful variable to change is 'methods', the set of
% recovery algorithms to compare.  It is defined below, after
% the definition of the recovery methods.

%
% Begin definitions of recovery methods
%

mattype = 'sparseplusminus16';

model.shortname = 'model';
model.name = 'Tree-based recovery + pyramid transformation';
model.MatrixGen = @(m) gen_matrix(pyramid.M, m, mattype);
model.RecoveryFn = @(m,x) pyramid.invPfun(recovery('ssmppostree(20,20)', pyramid.P*x, m, 300));

pyr.shortname = 'pospyramid';
pyr.name = 'Positive SSMP on pyramid';
pyr.MatrixGen = @(m) gen_matrix(pyramid.M, m, mattype);
pyr.RecoveryFn = @(m,x) pyramid.invPfun(recovery('ssmppos(20,20)', pyramid.P*x, m, 300));

pyr2.shortname = 'pyramid';
pyr2.name = 'Standard sparse recovery + pyramid transformation';
pyr2.MatrixGen = @(m) gen_matrix(pyramid.M, m, 'sparse16');
pyr2.RecoveryFn = @(m,x) pyramid.invPfun(recovery('ssmp(20,20)', pyramid.P*x, m, 300));

direct.shortname = 'ssmp';
direct.name = 'Standard sparse recovery';
direct.MatrixGen = @(m) gen_matrix(pyramid.N, m, 'sparse16');
direct.RecoveryFn = @(m,x) recovery('ssmp(20,20)', x, m, 300);

directp.shortname = 'posssmp';
directp.name = 'Positive SSMP on pixels';
directp.MatrixGen = @(m) gen_matrix(pyramid.N, m, mattype);
directp.RecoveryFn = @(m,x) recovery('ssmppos(20,20)', x, m, 300);

wavelet_type = 'haar';
s = [1 2.^[0:C.lgn]]'; s  = [s s];

wavelet.shortname = 'wavelet';
wavelet.name = 'SSMP on wavelets';
wavelet.MatrixGen = @(m) gen_matrix(pyramid.N, m, 'sparse16');
wavelet.RecoveryFn = @(m,x) waverec2(recovery('ssmp(20,20)', wavedec2(reshape(x, C.n_pixel, C.n_pixel), C.lgn, wavelet_type)', m, 300), s, wavelet_type);

waveletl1.shortname = 'waveletl1';
waveletl1.name = 'L1 on wavelets, sparse binary';
waveletl1.MatrixGen = @(m) gen_matrix(pyramid.N, m, 'sparse16');
waveletl1.RecoveryFn = @(m,x) waverec2(recovery('lp', wavedec2(reshape(x, C.n_pixel, C.n_pixel), C.lgn, wavelet_type)', m, 300), s, wavelet_type);

l1min.shortname = 'l1min';
l1min.name = 'L1 minimization of pixels, sparse binary';
l1min.MatrixGen = @(m) gen_matrix(pyramid.N, m, mattype);
l1min.RecoveryFn = @(m,x) recovery('lp', x, m);

l1min2.shortname = 'l1mingauss';
l1min2.name = 'L1 minimization of pixels, Gaussian';
l1min2.MatrixGen = @(m) gen_matrix(pyramid.N, m, 'gaussian');
l1min2.RecoveryFn = @(m,x) recovery('lp', x, m);

%
% End definitions of recovery methods
%

% This is the set of methods to actually perform and plot
methods = [model pyr pyr2 direct directp wavelet];

%
% Begin running the trials
%

trials = zeros(num_trials, numel(methods), numel(measurements), 3);
trials(:, :, 3, :) = 1;
result = zeros(num_trials, numel(methods), numel(measurements));
resultim = zeros(numel(measurements), numel(methods), C.n_pixel, C.n_pixel);

  if C.noise
    app = ' noisy';
  else
    app = '';
  end

for k = 1:num_trials
  if new_points 
    noiseless_image = gen_image('star', C.starwidth, C.numstars);
    noisy_image = poissrnd(noiseless_image);
    if C.noise
      noisy_image = abs(poissrnd(noiseless_image + C.fixednoise*ones(2^C.lgn)) + C.gaussnoise*randn(2^C.lgn));
    end
  end
  x = noisy_image(:);

  figure(1), display_image(noisy_image), colormap(bone), title(['Real image (' num2str(C.n_pixel) 'x' num2str(C.n_pixel), ', width ' num2str(C.starwidth) app ')'])
  for i = 1:numel(measurements)
    for j=1:numel(methods)
      mat = methods(j).MatrixGen(measurements(i));
      rec = methods(j).RecoveryFn(mat, x);
      recim = reshape(rec, C.n_pixel, C.n_pixel);
      resultim(i,j,:,:) = recim;
      trials(k,j,i,:) = calc_emd(noiseless_image, recim, 0);

      figure(j+1), display_image(recim)
      colormap(bone)
      title(get_title(measurements(i), methods(j).name))
    end
    result(k,:,i) = trials(k,:, i,1) ./ (1 - trials(k,:, i, 3)) / C.numstars;
    k
    squeeze(result(k,:,:))
  end
end

%
% End running the trials
%

%
% Begin plotting result
%

display_set=1:numel(methods);
%display_set = [1 3 4];    %To only plot a few results, uncomment this

median_result = squeeze(median(result, 1));

figure(numel(methods)+2)
clf
hold on
for i = display_set
  plot(measurements, median_result(i,:), '.-', 'DisplayName', methods(i).name, ...
       'Color', index(get(gca, 'ColorOrder'), i,:))
end
hold off
legend('toggle')
xlabel('Number of measurements')
ylabel('Average pixel error of star centroid')
title(['Centroid estimation error (' num2str(C.n_pixel) 'x' num2str(C.n_pixel), ', width ' num2str(C.starwidth) app ')'])

%
% End plotting result
%

%{
save 'moo.mat' 'resultim' 'result' 'measurements' 'methods' 'x' 'noiseless_image' 'noisy_image' 'C' 'median_result'

display_run(2, resultim, measurements, methods)

figure(2), imagesc(log(max(200,squeeze(resultim(1,1,:,:)))))

mat2 = gen_matrix(pyramid.M, 400, 'sparseplusminus16');
y2 = recovery('ssmppostree(30, 30)', Px, mat2, 300);
imagesc(min(invPfun(y2, C.lgn), 2000))
%}
