% generates a (+1, -1, 0) sparse matrix of M lines, N columns, and D +/-1s on each
% column 
% Written by Radu Berinde, MIT, Jan. 2008
% Revised by Eric Price, MIT, Oct. 2010

function matrix = gen_matrix_sparseplusminus(N, M, D, arg2_unused)

matrix.N = N;
matrix.M = M;
matrix.D = D;

matrix.neighbors = sort(uint32(randint(N, D, [1 M])), 2);
% Check for and fix duplicate neighbors 
for n = 1:N
  while true
    if (min(diff(sort(matrix.neighbors(n, :)))) > 0)
      break;
    end
    matrix.neighbors(n, :) = uint32(sort(randint(1, D, [1 M])));
  end
end

L = zeros(1, N*D);
C = zeros(1, N*D);

for n = 1:N
    for i = 1:D
        L((n-1)*D + i) = matrix.neighbors(n, i);
        C((n-1)*D + i) = n;
    end
end

V = sign(randn(1, N*D));
matrix.A = sparse(L, C, V, M, N); 

matrix.Afun  = @(z) matrix.A * z;
matrix.Atfun = @(z) matrix.A' * z;

matrix.MedianRecoveryFun = @(z) median_recovery_signed(matrix.A, z);
